/* Copyright (c) 2009 Ondrej Donek.
   See the file LICENSE.txt for licensing information. */
  
/**
 * Prototype object for single note.
 *
 * @param aArgs {mixed}
 * @type object
 */
function tbNotesNotePrototype(aArgs)
{
	if((aArgs instanceof Components.interfaces.mozIStorageStatementRow)) {
		this._id = aArgs.id;
		this._target_type = aArgs.target_type;
		this._target_key = aArgs.target_key;
		this._target_id = aArgs.target_id;
		this._target_name = aArgs.target_name;
		this._category_id = aArgs.category_id;
		this._created = aArgs.created;
		this._updated = aArgs.updated;
		this._title = aArgs.title;
		this._body = aArgs.body;
		this._deleted = ((aArgs.deleted == 1) ? true : false);
	}
};
tbNotesNotePrototype.prototype = 
{
  // id
  get mId() {
    return (typeof(this._id) != "undefined") ? this._id : null;
  },
  set mId(aVal) {
    delete this._id;
    
    this._id = aVal;
    this._need_update = true;
  },
  
  // target type
  get mTargetType() {
    return (typeof(this._target_type) != "undefined") ? this._target_type : null;
  },
  set mTargetType(aVal) {
    delete this._target_type;
    
    this._target_type = aVal;
    this._need_update = true;
  },
  
  // target key
  get mTargetKey() {
    return (typeof(this._target_key) != "undefined") ? this._target_key : '';
  },
  set mTargetKey(aVal) {
    delete this._target_key;
    
    this._target_key = aVal;
    this._need_update = true;
  },
  
  // target id
  get mTargetId() {
    return (typeof(this._target_id) != "undefined") ? this._target_id : '';
  },
  set mTargetId(aVal) {
    delete this._target_id;
    
    this._target_id = aVal;
    this._need_update = true;
  },
  
  // target name
  get mTargetName() {
    return (typeof(this._target_name) != "undefined") ? this._target_name : '';
  },
  set mTargetName(aVal) {
    delete this._target_name;
    
    this._target_name = aVal;
    this._need_update = true;
  },
  
  // category id
  get mCategoryId() {
    return (typeof(this._category_id) != "undefined") ? this._category_id : null;
  },
  set mCategoryId(aVal) {
    delete this._category_id;
    
    this._category_id = aVal;
    this._need_update = true;
  },
  
  // created
  get mCreated() {
    return (typeof(this._created) != "undefined") ? this._created : null;
  },
  set mCreated(aVal) {
    delete this._created;
    
    this._created = aVal;
    this._need_update = true;
  },
  
  // updated
  get mUpdated() {
    return (typeof(this._updated) != "undefined") ? this._updated : null;
  },
  set mUpdated(aVal) {
    delete this._updated;
    
    this._updated = aVal;
    this._need_update = true;
  },
  
  // title
  get mTitle() {
    return (typeof(this._title) != "undefined") ? this._title : null;
  },
  set mTitle(aVal) {
    delete this._title;
    
    this._title = aVal;
    this._need_update = true;
  },
  
  // body
  get mBody() {
    return (typeof(this._body) != "undefined") ? this._body : null;
  },
  set mBody(aVal) {
    delete this._body;
    
    this._body = aVal;
    this._need_update = true;
  },
  
  /**
   * Returns if note is marked as deleted or not.
   * 
   * @returns {boolean}
   */
  isMarkedAsDeleted: function() {
    return (typeof(this._deleted) != "undefined") ? this._deleted : false;
  },
  
  /**
   * Mark the note as deleted. Note's database record will be also updated.
   * 
   * @returns {void}
   */
  markDelete: function() {
    if(!this.isMarkedAsDeleted()) {
      this._deleted = true;
      
      if(this.mId != null) {
        tbNotesDb.deleteNote(this.mId);
      }
    }
  },
  
  /**
   * Ensure that the note is not marked as deleted. Note's database record will 
   * be also updated.
   * 
   * @returns {void}
   */
  unmarkDelete: function() {
    if(this.isMarkedAsDeleted()) {
      this._deleted = false;
      
      if(this.mId != null) {
        tbNotesDb.undeleteNote(this.mId);
      }
    }
  },
  
  /**
   * Remove note permanently (from the storage, object remains 
   * how long you will hold it (only id is deleted so mId is null).
   * 
   * @returns {boolean}
   */
  remove: function() {
    if(this.mId != null) {
      if(!tbNotesDb.removeNote(this.mId)) {
        return false;
      }
    }
    
    delete this._id;
    return true;
  },
  
  /**
   * Returns TRUE if object was changed but not saved.
   *
   * @returns {boolean}
   */
  needSave: function() {
    if(typeof(this._need_update) == "undefined") {
      return false;
    }
    
    return this._need_update;
  },
  
  /**
   * Save/update note's record in the storage. If note hasn't been saved yet 
   * (so id is null) will be note added into the storage and id updated.
   * 
   * @returns {bolean}
   */
  save: function() {
    if(typeof(this._need_update) != "undefined") {
      delete this._need_update;
    }
    
    var res = false;
    var links = this.getLinks();
    
    if(this.mId == null) {
      // We are saving new note
      var newId = tbNotesDb.addNote(this);
      
      if(newId != null) {
        this._id = newId;
        res = true;
      } else {
        return false;
      }
      
      // Update links
      for(var link in links) {
        // Ensure that all links has attached correct note ID
        links[link].mNoteId = this.mId;
        links[link].save();
      }
    } else {
      // We are saving already existing note
      res = tbNotesDb.updateNote(this);
      
      for(var link in links) {
        if(typeof(links[link].mId) != "undefined" && links[link].mId != null) {
          links[link].save();
        } else {
          links[link].mNoteId = this.mId;
          links[link].save();
        }
      }
    }
    
    // Ensure that links will be next time loaded from database
    delete this._links;
    
    return res;
  },
  
  /**
   * Sets links that are attached to the note. All currently attached links
   * will be removed.
   *
   * @param aLinks {array}
   * @returns {void}
   */
  setLinks: function(aLinks) {
    // TODO Check if aLinks is an array
    this._links = aLinks;
    this._need_udate = true;
  },
  
  /**
   * Returns links that are attached to the note.
   *
   * @returns {array} Array of {tbNotesLinkPrototype}
   */
  getLinks: function() {
    if(typeof(this._links) == "undefined") {
      this._links = tbNotesDb.getLinksByNoteId(this.mId);
    }
    
    return this._links;
  },
  
  /**
   * Returns object as a string.
   * 
   * @return {string}
   */
  toString: function() {
    var ret = "{ id: \"" + this.mId + "\", " +
              "target_type: \"" + this.mTargetType + "\", " +
              "target_key: \"" + this.mTargetKey + "\", " +
              "target_id: \"" + this.mTargetId + "\", " +
              "target_name: \"" + this.mTargetName + "\", " +
              "category_id: \"" + this.mCategoryId + "\", " +
              "created: \"" + this.mCreated + "\", " + 
              "updated: \"" + this.mUpdated + "\", " + 
              "title: \"" + this.mTitle + "\", " + 
              "body: \"" + this.mBody + "\", " +
              "deleted: \"" + ((this.isMarkedAsDeleted()) ? "Yes" : "No") + "\", " + 
              "links: { ";
    
    var links = this.getLinks();
    for(var link in links) {
      ret = ret + links[link].toString() + ", ";
    }
    
    ret = ret + "}";
    
    return ret;
  }
  
}; // End of tbNotesNotePrototype
