/* Copyright (c) 2009 Ondrej Donek.
   See the file LICENSE.txt for licensing information. */

/**
 * Here is object that holds some basic utility methods.
 *
 * @type Object
 */
var tbNotesUtils =
{
  
	/**
	 * Helper function that returns current date and time (e.g. "2009-09-16 14:33:00").
	 * 
	 * @return string
	 */
	getCurrentDatetime: function() 
	{
		var d = new Date();
		var r = d.getFullYear() + "-" + 
						((d.getMonth()<10) ? "0" + d.getMonth() : d.getMonth()) + "-" + 
						((d.getDate()<10) ? "0" + d.getDate() : d.getDate()) + " " + 
						((d.getHours()<10) ? "0" + d.getHours() : d.getHours()) + ":" + 
						((d.getMinutes()<10) ? "0" + d.getMinutes() : d.getMinutes()) + ":00";
		
		return r;
	}, // end getCurrentDatetime()
	
	/**
	 * Removes all child nodes of given element.
	 * 
	 * @param aElm DOMElement
	 * @return void
	 */
	removeChilds: function(aElm)
	{
		try {
			var ch = aElm.lastChild;
			while(ch) {
				aElm.removeChild(ch);
				ch = aElm.lastChild;
			}
		} catch(e) {
			Components.utils.reportError(e);
		}
	}, // end removeChilds(aElm)

	/**
	 * Copy given text to the system clipboard
	 *
	 * @param aText {string}
	 * @returns {boolean}
	 */
	copyTextToClipboard: function(aText)
	{
		try {
			var clip = Components.classes["@mozilla.org/widget/clipboard;1"].createInstance(Components.interfaces.nsIClipboard);
			var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
			var str = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString);
			
			str.data = aText;
			trans.addDataFlavor("text/unicode");
			trans.setTransferData("text/unicode", str, aText.length * 2);
			clip.setData(trans, null, Components.interfaces.nsIClipboard.kGlobalClipboard);

			return true;
		} catch(e) {
			Components.utils.reportError(e);
			
			return false;
		}
	}, // end copyTextToClipboard(aText)
	
	// TODO Implement another method for opening links (according to user's preferences)
	openLinkInInternalBrowser: function(aUrl)
	{
		// ...
	}, // end openLinkInInternalBrowser(aUrl)
	
	/**
	 * Opens link in the system browser.
	 *
	 * @param aUrl {string}
	 */
	openLinkInExternalBrowser: function(aUrl)
	{
		try {
			var ioSrv = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
			var uri = ioSrv.newURI(aUrl, null, null);
			var extps = Components.classes["@mozilla.org/uriloader/external-protocol-service;1"].
					getService(Components.interfaces.nsIExternalProtocolService);

			extps.loadURI(uri, null);
		} catch(e) {
			Components.utils.reportError(e);
		}
	} // end openLinkInExternalBrowser(aUrl)
  
}; // End of tbNotesUtils
