/* Copyright (c) 2009 Ondrej Donek.
   See the file LICENSE.txt for licensing information. */

/**
 * Controller object for tbNotes address book overlay.
 * 
 * @type Object
 */
var tbNotesAbOverlay =
{
  
  /**
   * On load - check if user want to use tbNotes in the address book and if yes
   * enable it.
   *
   * @param aEvent {Components.interfaces.nsIDOMEvent}
   * @returns {void}
   */
  onLoad: function()
  {
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].
						getService(Components.interfaces.nsIPrefBranch);
    var enable = prefs.getBoolPref("extensions.tbnotes.addressbook_allow_tbnotes");
    
    var mainmenu = document.getElementById("tbnotes-menu");
    var toolbarbtn1 = document.getElementById("tbn-addnote-toolbarbutton");
    var toolbarbtn2 = document.getElementById("tbn-shownotesview-toolbarbutton");
    
    tbNotesAbOverlay.showHideElm(mainmenu, enable);
    tbNotesAbOverlay.showHideElm(toolbarbtn1, enable);
    tbNotesAbOverlay.showHideElm(toolbarbtn2, enable);
  }, // end onLoad()
	
	/**
	 * Performs command with given identifier.
	 *
	 * @param aCommand {string}
	 * @returns {void}
	 */
	doCommand: function(aCommand)
	{
		switch(aCommand) {
			case "show_prefs":
			case "show_about":
			case "show_notesview":
				with(tbNotesAbOverlay.getMessengerWin()) {
					tbNotes.doCommand(aCommand, "addressbook");
				}
				break;
			
			case "add_note":
			case "edit_note":
			case "remove_note":
				alert("Command [" + aCommand + "] is not implemented yet!");
				break;
		}
	},
	
	/**
	 * Returns object of the main Thunderbird's window
	 *
	 * @return {Components.interfaces.nsIXULWindow}
	 */
	getMessengerWin: function()
	{
		var noteObj = null;
		var winMan = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService();
		var winManInt = winMan.QueryInterface(Components.interfaces.nsIWindowMediator);
		
		return winManInt.getMostRecentWindow("mail:3pane");
	}, // end getMessengerWin()
  
  /**
   * Hide/show element.
   *
   * @param aElement {Components.interfaces.nsIDOMElement}
   * @param aShow {boolean}
   * @returns {void}
   */
  showHideElm: function(aElement, aShow)
	{
    if(aElement) {
      if(aShow === true) {
        if(aElement.hasAttribute("hidden")) {
          aElement.removeAttribute("hidden");
        }
      } else {
        aElement.setAttribute("hidden", "true");
      }
    }
  } // end showHideElm(aElement, aShow)
  
}; // End of tbNotesAbOverlay


// ============================================================================
// Add on load event listener

window.addEventListener("load", tbNotesAbOverlay.onLoad, false);
